@@ -1,8 +1,8 @@
 'ursaProgressBar' <- function(kind=c("tk","txt")
                            # ,title=basename(strsplit(commandArgs(FALSE)[4],"=")[[1]][2])
-                             ,title=.argv0()
-                             ,label=""
-                             ,min=0,max=1,initial=min,width=NA,style=1,silent=FALSE) {
+                             ,title=.argv0(),label=""
+                             ,min=0,max=1,initial=min,width=NA,style=1
+                             ,tail=FALSE,silent=FALSE) {
    if (silent) {
       pb <- logical()
       class(pb) <- "ursaProgressBar"
@@ -18,17 +18,19 @@
    else
       kind <- kind0
    if (kind=="tk") {
+      Rver <- R.Version()
       if ((.Platform$OS.type=="unix")&&(!nchar(Sys.getenv("DISPLAY"))))
          kind <- "txt"
       else if ((!requireNamespace("tcltk",quietly=.isPackageInUse()))||
-        (!capabilities("tcltk")))
+        (!capabilities("tcltk"))||((Rver$'svn rev' %in% c("78619"))&&(Rver$arch %in% c("x86_64"))))
          kind <- "txt"
    }
    if ((!is.na(title))&&(!nchar(title)))
       title <- "'ursa' package"
    t1 <- proc.time()[3]
    names(t1) <- title
-   st <- c(t1,prev=unname(t1),delta=0,min=min,max=max,current=initial)
+   st <- c(t1,prev=unname(t1),delta=0,min=min,max=max,current=initial
+          ,tail=as.numeric(tail))
    if (kind=="tk") {
       if (is.na(width))
          width <- 360
@@ -56,16 +58,18 @@
    op <- list(st)
    names(op) <- pb$optionName
    options(op)
-   setUrsaProgressBar(pb)
+   class(pb) <- c("ursaProgressBar",class(pb))
+   if (tail)
+      setUrsaProgressBar(pb)
    pb
 }
 'setUrsaProgressBar' <- function(pb,value,title=NULL,label=NULL) {
-   if (inherits(pb,"ursaProgressBar"))
+   if ((length(class(pb))==1)&&(inherits(pb,"ursaProgressBar")))
       return(pb)
   # if (.isKnitr())
   #    return(pb)
    t2 <- unname(proc.time()[3])
-   cl <- class(pb)
+   cl <- class(pb)[2]
    st <- getOption(pb$optionName)
    if (missing(value))
       st["current"] <- st["current"]+1
@@ -83,16 +87,21 @@
    op <- list(st)
    names(op) <- pb$optionName
    options(op)
-   if (!reset)
+   if ((!reset)&&(T & st["max"]!=st["current"]))
       return(NULL)
    if (missing(value))
       value <- unname(st["current"])-1
    ##~ if (st["current"]>st["max"])
       ##~ st["current"] <- st["max"]
-   if (value>=st["max"])
-      value <- unname(st["max"])-1
+  # if (value>=st["max"])
+  #    value <- unname(st["max"])-1
   # print(c(st,value=value))
-   title <- sprintf("%s: %.0f of %.0f",names(st)[1],value+1,st["max"])
+   if (value<st["max"])
+      title <- sprintf("%s: %.0f of %.0f",names(st)[1],value+1,st["max"])
+   else {
+      title <- sprintf("%s: %s!",names(st)[1]
+          ,ifelse(isTRUE(all.equal(value,unname(st["max"]))),"completed","OVER LIMIT"))
+   }
    d1 <- unname((value-st["min"])/(st["max"]-st["min"]))
    t3 <- t2-st[1]
   # print(cbind(as.data.frame(t(st)),onset=reset,curr=t3))
@@ -128,7 +137,7 @@
       d8 <- (d8-d8s)/60
       d8m <- d8%%60
       d8h <- (d8-d8m)/60
-      d8 <- sprintf("%02d:%02d:%02d",d8h,d8m,d8s)
+     # d8 <- sprintf("%02d:%02d:%02d",d8h,d8m,d8s)
       d9s <- d9%%60
       d9 <- (d9-d9s)/60
       d9m <- d9%%60
@@ -137,6 +146,10 @@
          d9 <- sprintf("%02d:%02d:%02d",d9h,d9m,d9s)
       else
          d9 <- sprintf("%02dh%02dm",d9h,d9m)
+      if (d8h==0)
+         d8 <- sprintf("%02d:%02d:%02d",d8h,d8m,d8s)
+      else
+         d8 <- sprintf("%02dh%02dm",d8h,d8m)
       d8 <- paste0("+",d8)
       d4f <- paste0("-",d4f)
      # d9 <- paste0("=",d9)
@@ -146,6 +159,7 @@
         # label <- sprintf("%5.0f%% ???:??? (????-??-?? ??:??)",d1*100)
          label <- sprintf("%5.0f%% %30s",d1*100," ")
    }
+  # print(c(value=value))
    if (cl=="tkProgressBar")
       return(tcltk::setTkProgressBar(pb,value,title=title,label=label))
    ##~ else if ((cl=="winProgressBar")&&(.Platform$OS.type=="windows"))
@@ -153,6 +167,12 @@
    utils::setTxtProgressBar(pb,value,title=title,label=label)
 }
 'close.ursaProgressBar' <- function(con,...) {
-   invisible(NULL)
+   if (!length(con))
+      return(invisible(NULL))
+   p <- getOption(con$optionName)
+   if (p["current"]==p["max"]) {
+      setUrsaProgressBar(con,unname(p["max"]))
+   }
+   NextMethod("close",con,...)
 }
 # 'close.ursaProgressBar' <- function(con,...) close(con,...)
