@@ -319,15 +319,17 @@
             }
            # else
            #    obj <- NULL
-            if (limLonLat)
+            if (limLonLat) {
                sp::proj4string(obj) <- sp::CRS("+init=epsg:4326")
+            }
             else if (!is.null(proj4)) {
-               sp::proj4string(obj) <-  sp::CRS(proj4)
+               sp::proj4string(obj) <-  sp::CRS(proj4,doCheckCRSArgs=FALSE)
                style <- proj4
                session_grid(NULL)
             }
-            else
-               sp::proj4string(obj) <- sp::CRS(session_proj4())
+            else {
+               sp::proj4string(obj) <- sp::CRS(session_proj4(),doCheckCRSArgs=FALSE)
+            }
          }
          else if (inherits(dsn,"data.frame")) {
             obj <- dsn
@@ -514,7 +516,7 @@
             if (isSP) {
                sp::coordinates(obj) <- ~x+y
                if (!is.null(p4s))
-                  sp::proj4string(obj) <- sp::CRS(p4s)
+                  sp::proj4string(obj) <- sp::CRS(p4s,doCheckCRSArgs=FALSE)
             }
             hasOpened <- TRUE
            # display(a)
@@ -775,7 +777,7 @@
    }
    projClass <- c("longlat","stere","laea","merc")
    projPatt <- paste0("(",paste(projClass,collapse="|"),")")
-   staticMap <- c("openstreetmap","google","sputnikmap")
+   staticMap <- c("openstreetmap","sputnikmap","google")
    tilePatt <- paste0("(",paste0(unique(c(staticMap,.tileService())),collapse="|"),")")
    retina <- getOption("ursaRetina",1)
    len <- 640L # as.integer(round(640*getOption("ursaRetina",1)))
@@ -857,10 +859,7 @@
   # isWeb <- isOSM | isGoogle | tryTile
    if ((is.null(g0))||(is.numeric(lon0))||(is.numeric(lat0))) {
   # if ((resetProj)||(is.ursa(g0,"grid"))||(is.numeric(lon0))||(is.numeric(lat0))) {
-      if (isSF)
-         proj4 <- sf::st_crs(obj)$proj4string
-      if (isSP)
-         proj4 <- sp::proj4string(obj)
+      proj4 <- spatial_crs(obj)
       if ((is.na(proj4))&&(nchar(style))&&(.lgrep("\\+proj=.+",style))) { ## ++ 20180530
          proj4 <- style
         # isPROJ4 <- FALSE
@@ -1043,6 +1042,10 @@
          }
          else if (proj=="laea") {
             if (style=="polarmap") {
+               if (length(crs)) {
+                  lon_0 <- as.numeric(gsub(".*\\+lon_0=(\\S+)\\s.*","\\1"
+                                          ,spatial_crs(crs)))
+               }
                lon_0[lon_0<(-165) || lon_0>=(+135)] <- -180
                lon_0[lon_0>=(-165) && lon_0<(-125)] <- -150
                lon_0[lon_0>=(-125) && lon_0<(-70)] <- -100
@@ -1056,7 +1059,7 @@
                           ,"+k=1","+x_0=0 +y_0=0 +ellps=WGS84 +units=m +no_defs")
          }
          else if (proj=="merc")
-            t_srs <- paste("","+proj=merc +a=6378137 +b=6378137"
+            t_srs <- paste("+proj=merc +a=6378137 +b=6378137"
                           ,"+lat_ts=0.0",paste0("+lon_0=",lon_0)
                           ,"+x_0=0.0 +y_0=0 +k=1.0 +units=m"
                          # ,"+nadgrids=@null"
@@ -1066,11 +1069,11 @@
          }
          else if (proj %in% c("zzzgoogle")) {
             if (FALSE)#(selection %in% c(1000L,3L))
-               t_srs <- paste("","+proj=merc +a=6378137 +b=6378137"
+               t_srs <- paste("+proj=merc +a=6378137 +b=6378137"
                              ,"+lat_ts=0.0 +lon_0=180.0 +x_0=0.0 +y_0=0 +k=1.0"
                              ,"+units=m +nadgrids=@null +wktext +no_defs")
             else
-               t_srs <- paste("","+proj=merc +a=6378137 +b=6378137"
+               t_srs <- paste("+proj=merc +a=6378137 +b=6378137"
                              ,"+lat_ts=0.0 +lon_0=0.0 +x_0=0.0 +y_0=0 +k=1.0"
                              ,"+units=m +nadgrids=@null +wktext +no_defs")
          }
@@ -1103,7 +1106,8 @@
                }
             }
             if (isSP) {
-               obj <- sp::spTransform(obj,t_srs)
+              # obj <- sp::spTransform(obj,t_srs)
+               obj <- spatial_transform(obj,t_srs)
                if ((TRUE)&&(.lgrep("\\+proj=longlat",t_srs))&&(max(lon2)>180)) {
                   if (verbose)
                      .elapsedTime("lon+360 -- start")
@@ -1156,6 +1160,9 @@
            # str(style)
            # str(t_srs)
             if (isSF) {
+               patt <- "\\+init=epsg\\:(.+)\\s*$"
+               if (length(grep(patt,t_srs)))
+                  t_srs <- as.integer(gsub(patt,"\\1",t_srs))
               # str(sf::st_crs(obj))
                obj <- sf::st_transform(obj,t_srs)
             }
@@ -1182,17 +1189,36 @@
         # opE <- options(show.error.messages=TRUE)
         # print(sf::st_bbox(obj))
          src0 <- sf::st_crs(obj)$proj4string
-         if (!is.na(src0))
-            obj <- sf::st_transform(obj,t_srs)
+         if (!is.na(src0)) {
+            t_srs <- spatial_crs(t_srs)
+            g0$proj4 <- t_srs
+            if (!identical(src0,t_srs))
+               obj <- sf::st_transform(obj,t_srs)
+         }
         # print(sf::st_crs(obj)$proj4string)
         # print(sf::st_bbox(obj))
         # options(opE)
       }
       if (isSP) {
-         src0 <- sp::proj4string(obj)
+         if (FALSE)
+            src0 <- sp::proj4string(obj)
+         else if (FALSE) {
+            src0 <- methods::slot(obj,"proj4string")
+            if (methods::is(src0,"CRS"))
+               src0 <- methods::slot(src0,"projargs")
+         }
+         else
+            src0 <- spatial_crs(obj)
         # print(c(sp::bbox(obj)))
-         if (!is.na(src0))
-            obj <- sp::spTransform(obj,t_srs) ## not tested
+         if ((!is.na(src0))&&(T | !identical(src0,t_srs))) {
+            if (.lgrep("\\+init=epsg",t_srs)) {
+               t_srs <- .epsg2proj4(t_srs,force=TRUE)
+               if (.lgrep("\\+init=epsg",src0))
+                  src0 <- .epsg2proj4(src0,force=TRUE)
+               if (!identical(src0,t_srs))
+                 obj <- sp::spTransform(obj,t_srs) ## not tested
+            }
+         }
         # print(sp::proj4string(obj))
         # print(c(sp::bbox(obj)))
       }
@@ -1262,7 +1288,13 @@
       res <- with(g0,sqrt(resx*resy))
       s <- 2*6378137*pi/(2^(1:21+8))
       zoom <- which.min(abs(s-res))
-      g0 <- regrid(g0,res=s[zoom])
+      if ((style=="polarmap")&&(zoom>9)) {
+         znew <- 9
+         g0 <- regrid(g0,res=s[znew],expand=ifelse(dev <-F ,2^(zoom-znew),1))
+         zoom <- znew
+      }
+      else
+         g0 <- regrid(g0,res=s[zoom])
    }
    if (any(border!=0)) {
       g0 <- regrid(g0,border=border)
